@@ -719,51 +719,6 @@ cdrecord_stderr_line (NautilusBurnProcess   *process,
 	return TRUE;
 }
 
-static gboolean
-media_type_matches (NautilusBurnMediaType type,
-		    gboolean              is_blank,
-		    gboolean              was_unmounted)
-{
-	if (was_unmounted) {
-		switch (type) {
-		case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-		case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-		case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-			return TRUE;
-		default:
-			return FALSE;
-		}
-	}
-
-	switch (type) {
-	case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
-	case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
-		return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
-		return TRUE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CD:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
-		return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CDR:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
-		if (is_blank)
-			return TRUE;
-		else
-			return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-		return TRUE;
-	default:
-		break;
-	}
-
-	return FALSE;
-}
-
 static NautilusBurnMediaType
 nautilus_burn_recorder_wait_for_insertion (NautilusBurnRecorder *recorder,
 					   NautilusBurnDrive    *drive,
@@ -771,68 +726,64 @@ nautilus_burn_recorder_wait_for_insertion (NautilusBurnRecorder *recorder,
 					   gboolean             *blank)
 {
 	NautilusBurnMediaType type;
+	gint64                media_size;
 	gboolean              reload;
-	gboolean              last_was_unmount;
 	gboolean              is_rewritable;
+	gboolean              can_write;
 	gboolean              is_blank;
+	gboolean              is_mounted;
 	gboolean              has_data;
 	gboolean              has_audio;
 
-	last_was_unmount = FALSE;
 	reload = FALSE;
+
+ again:
+
+	/* if drive is mounted then unmount before checking anything */
+	if (nautilus_burn_drive_is_mounted (drive)) {
+		nautilus_burn_drive_unmount (drive);
+	}
+
+	media_size = nautilus_burn_drive_get_media_size (drive);
 	type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
+	can_write = nautilus_burn_drive_media_type_is_writable (type, is_blank);
 
-	if (can_rewrite)
+	if (can_rewrite) {
 		*can_rewrite = is_rewritable;
+	}
 
-	if (blank)
+	if (blank) {
 		*blank = is_blank;
+	}
 
-	last_was_unmount = FALSE;
-	while (!media_type_matches (type, is_blank, last_was_unmount)) {
-		gboolean res, busy_cd;
-
-		busy_cd = (type == NAUTILUS_BURN_MEDIA_TYPE_BUSY);
-
-		if (busy_cd && !last_was_unmount) {
-			/* umount the disc if we find it, and try it again */
-			if (nautilus_burn_drive_unmount (drive)) {
-				last_was_unmount = TRUE;
-				goto retry;
-			}
-		}
+	if (! can_write) {
+		gboolean is_mounted;
+		int      res;
 
+		reload = (media_size > 0);
 		if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
 			reload = FALSE;
 		}
 
+		/* check one more time */
+		is_mounted = nautilus_burn_drive_is_mounted (drive);
+
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [INSERT_MEDIA_REQUEST],
-			       0, reload, recorder->priv->can_rewrite,
-			       busy_cd, &res);
+			       0,
+			       reload,
+			       recorder->priv->can_rewrite,
+			       is_mounted,
+			       &res);
 
 		if (res == FALSE) {
 			return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		}
 
-		last_was_unmount = FALSE;
-
-	retry:
-		type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
-
-		if (can_rewrite)
-			*can_rewrite = is_rewritable;
-
-		if (blank)
-			*blank = is_blank;
-
-		reload = FALSE;
-		if (type == NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN) {
-			reload = TRUE;
-		}
+		goto again;
 	}
 
-	if (!is_blank) {
+	if (! is_blank) {
 		int res;
 
 		res = 0;
@@ -843,7 +794,7 @@ nautilus_burn_recorder_wait_for_insertion (NautilusBurnRecorder *recorder,
 			       0, &res);
 
 		if (res == NAUTILUS_BURN_RECORDER_RESPONSE_RETRY) {
-			goto retry;
+			goto again;
 		} else if (res == NAUTILUS_BURN_RECORDER_RESPONSE_CANCEL) {
 			type = NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		}
@@ -1366,6 +1317,7 @@ nautilus_burn_recorder_write_growisofs (NautilusBurnRecorder          *recorder,
 	g_free (speed_str);
 	g_free (dev_str);
 	g_ptr_array_free (argv, TRUE);
+	argv = NULL;
 	
 	g_signal_emit (recorder,
 		       nautilus_burn_recorder_table_signals [ANIMATION_CHANGED], 0,
@@ -1598,6 +1550,7 @@ nautilus_burn_recorder_write_cdrecord (NautilusBurnRecorder          *recorder,
 	g_free (speed_str);
 	g_free (dev_str);
 	g_ptr_array_free (argv, TRUE);
+	argv = NULL;
 
 	return result;
 }
