@@ -1014,50 +1014,6 @@ select_iso_filename (const char *default_name)
 	return filename;
 }
 
-static gboolean
-media_type_matches (NautilusBurnMediaType type,
-		    gboolean              is_blank,
-		    gboolean              was_unmounted)
-{
-	if (was_unmounted) {
-		switch (type) {
-		case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-		case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-		case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-			return TRUE;
-		default:
-			return FALSE;
-		}
-	}
-
-	switch (type) {
-	case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
-	case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
-		return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
-		return TRUE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CD:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
-		return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CDR:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
-		if (is_blank)
-			return TRUE;
-		else
-			return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-		return TRUE;
-	default:
-		break;
-	}
-
-	return FALSE;
-}
-
 static gboolean
 request_media (NautilusBurnDrive *drive,
 	       gint64             required_size,
@@ -1066,81 +1022,66 @@ request_media (NautilusBurnDrive *drive,
 {
 	gint64                media_size;
 	gboolean              reload;
-	gboolean              last_was_unmount;
 	NautilusBurnMediaType type;
 	gboolean              is_rewritable;
+	gboolean              can_write;
 	gboolean              is_blank;
 	gboolean              has_data;
 	gboolean              has_audio;
 
-	last_was_unmount = FALSE;
-	reload           = FALSE;
-
-	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE)
+	/* If the output is a file no need to prompt */
+	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		return TRUE;
+	}
 
-	if (request_overburn)
+	reload = FALSE;
+
+ again:
+
+	/* if drive is mounted then unmount before checking anything */
+	if (nautilus_burn_drive_is_mounted (drive)) {
+		nautilus_burn_drive_unmount (drive);
+	}
+
+	/* if overburn requested then no need to check size/type */
+	if (request_overburn) {
 		return TRUE;
+	}
 
 	media_size = nautilus_burn_drive_get_media_size (drive);
-
 	type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
+	can_write = nautilus_burn_drive_media_type_is_writable (type, is_blank);
 
-	while ((!media_type_matches (type, is_blank, last_was_unmount))
-	       || (required_size > media_size)) {
-		gboolean              busy_cd;
-		gboolean              request_overburn;
-		int                   res;
-
-		busy_cd = (type == NAUTILUS_BURN_MEDIA_TYPE_BUSY);
-
-		if (busy_cd && !last_was_unmount) {
-			/* umount the disc if we find it, and try it again */
-			if (nautilus_burn_drive_unmount (drive)) {
-				last_was_unmount = TRUE;
-				goto retry;
-			}
-		}
+	if ((! can_write) || (required_size > media_size)) {
+		gboolean suggest_overburn;
+		int      res;
 
 		reload = (media_size > 0);
-
-		if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR)
+		if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
 			reload = FALSE;
+		}
 
-		request_overburn = ((required_size - media_size) <= SUGGEST_OVERBURN);
+		suggest_overburn = ((required_size - media_size) <= SUGGEST_OVERBURN);
 
 		res = ask_for_media (drive,
 				     reload,
 				     FALSE,
-				     request_overburn,
+				     suggest_overburn,
 				     required_size);
 
-		if (res == GTK_RESPONSE_CANCEL)
+		if (res == GTK_RESPONSE_CANCEL) {
 			return FALSE;
+		}
 
 		if (res == RESPONSE_OVERBURN) {
-			if (use_overburn)
+			if (use_overburn) {
 				*use_overburn = TRUE;
-			break;
+			}
+
+			return TRUE;
 		}
 
-		last_was_unmount = FALSE;
-
-	retry:
-		type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
-
-		reload = FALSE;
-		if (type == NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN) {
-			reload = TRUE;
-		}
-
-		media_size = nautilus_burn_drive_get_media_size (drive);
-
-	}
-
-	if (!is_blank) {
-		/* don't ask user for confirmation of blank here, wait for warn-data-loss signal */
-		/*return ask_rewrite_disc (drive);*/
+		goto again;
 	}
 
 	return TRUE;
